/* 
 * eclipse-winscp-upload-script
 * Copyright (C) 2012  Stefan Kleeschulte
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 * Additional information about this script is available under the following
 * URL:
 * http://rays-blog.de/2012/05/05/94/use-winscp-to-upload-files-using-eclipses-autobuild-feature/
 */


/*******************************************************************************
*  C O N F I G U R A T I O N                                                   *
*******************************************************************************/

/*
 * -----------------------------------------------------------------------------
 * dir_assignment
 * -----------------------------------------------------------------------------
 * 
 * Assignment of local directories to remote directories. Local directories are
 * relative to the project root. Use (back-)slashes as shown in the examples.
 * 
 * If for a folder there is one assignment, and for one of its parent folders
 * there is another assignment, it will always be used the most specific
 * assignment that fits to a path.
 * 
 * Examples:
 * 
 * var dir_assignment = [
 *   ["\\files","/files/project"],
 *   ["\\htdocs","/htdocs/project"]
 * ];
 * 
 * var dir_assignment = [
 *   ["\\subfolder1\\subfolder2","/remote/path"]
 * ];
 * 
 * var dir_assignment = [
 *   ["\\subfolder1","/remote/path1"],
 *   ["\\subfolder1\\subfolder2","/remote/path2"]
 * ];
 */

var dir_assignment = [
  ["\\files","/files/project"],
  ["\\htdocs","/htdocs/project"]
];

/*
 * -----------------------------------------------------------------------------
 * winscp_com
 * -----------------------------------------------------------------------------
 * 
 * Path to WinSCP.com, relative to the "Base Directory" specified in the
 * builders launch configuration
 * 
 * Examples:
 * 
 * var winscp_com = 'C:\\Program Files (x86)\\WinSCP\\WinSCP.com';
 * var winscp_com = 'WinSCP.com';
 */

var winscp_com = 'WinSCP.com';

/*
 * -----------------------------------------------------------------------------
 * winscp_open_arguments
 * -----------------------------------------------------------------------------
 * 
 * Arguments for WinSCP's open command. These specify the connection settings.
 * Supported Protocols are FTP, FTPS (FTP-SSL), SFTP and SCP.
 * 
 * See WinSCP's scripting documentation for an overview of available arguments:
 * http://winscp.net/eng/docs/scriptcommand_open
 * 
 * A password needs to be specified in cleartext here. However, as long as the
 * connection is specified in the format [protocol]://[user]:[password]@[server]
 * the password will be starred out in the console output.
 * 
 * Examples:
 * 
 * FTP: 
 * var winscp_open_arguments = 'ftp://user:password@server.tld';
 * 
 * FTPS:
 * var winscp_open_arguments = 'ftps://user:password@server.tld -explicitssl -certificate="00:11:22:33:44:55:66:77:88:99:aa:bb:cc:dd:ee:ff:a0:b1:c2:d3"';
 * (To find out the certificate's fingerprint, connect to the server using
 * WinSCP GUI (WinSCP.exe) and go to Commands -> Server/Protocol Information.)
 */

var winscp_open_arguments = 'ftp://user:password@server.tld';

/*
 * -----------------------------------------------------------------------------
 * synchronize_on_autobuild_failure
 * -----------------------------------------------------------------------------
 * 
 * Boolean variable, determines whether on an autobuild failure a full sync
 * shall be performed. Explanation: When build.js is triggered by eclipses
 * autobuild feature, only added, changed and removed files are passed to
 * WinSCP by the script. If for some reason WinSCP fails to execute the given
 * commands (meaning WinSCP's exit code is not 0), the script can
 * automatically perform a full sync of the specified directories using
 * WinSCP's synchronize feature.
 * 
 * Examples:
 * 
 * var synchronize_on_autobuild_failure = true; // default
 * var synchronize_on_autobuild_failure = false;
 * 
 */

var synchronize_on_autobuild_failure = true;

/*
 * -----------------------------------------------------------------------------
 * winscp_console_mode
 * -----------------------------------------------------------------------------
 * 
 * Normally this script uses StdIn and StdOut to control WinSCP. This is faster
 * as no temporary WinSCP batch file has to be created. However, if you
 * encounter problems you might want to try setting this to true.
 * 
 * Examples:
 * 
 * var winscp_console_mode = false; // default
 * var winscp_console_mode = true;
 */

var winscp_console_mode = false;

/*
 * -----------------------------------------------------------------------------
 * explicitly_delete_folder_contents
 * -----------------------------------------------------------------------------
 * 
 * Normally, when a non empty folder is removed, only one command for deletion
 * of this folder is passed to WinSCP. However, if this for some reason does not
 * work, you can have this script pass explicit commands to WinSCP to remove all
 * subfolders and files first.
 * 
 * Examples:
 * 
 * var explicitly_delete_folder_contents = false; // default
 * var explicitly_delete_folder_contents = true;
 */
var explicitly_delete_folder_contents = false;



/*******************************************************************************
*  S C R I P T                                                                 *
*******************************************************************************/

function autobuild(){var a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s={};s.added=[],s.changed=[],s.removed=[],s.files=[],s.dirs=[],s.added_files=[],s.added_dirs=[],s.changed_files=[],s.changed_dirs=[],s.removed_files=[],s.removed_dirs=[];if(WScript.Arguments.length<4)WScript.Echo("Wrong arguments - no paths or sourcefile specified."),WScript.Quit(1);else if(WScript.Arguments.Item(2)=="-s"){a=WScript.Arguments.Item(3),b=new ActiveXObject("Scripting.FilesystemObject");if(b.FileExists(a)){c=b.OpenTextFile(a,1,!1);while(!c.AtEndOfStream){d=c.ReadLine(),e=d.substr(0,1),f=d.substring(3,d.length-1),f=f.split('" "');switch(e){case"a":s.added=s.added.concat(f);break;case"c":s.changed=s.changed.concat(f);break;case"r":s.removed=s.removed.concat(f);break;case"f":s.files=s.files.concat(f);break;case"d":s.dirs=s.dirs.concat(f)}}c.Close()}else WScript.Echo('Sourcefile "'+a+'" does not exist.'),WScript.Quit(1)}else{g="";for(h=2;h<WScript.Arguments.length;h++)switch(WScript.Arguments.Item(h)){case"-a":g="added";break;case"-c":g="changed";break;case"-r":g="removed";break;case"-f":g="files";break;case"-d":g="dirs";break;default:g!=""?s[g].push(WScript.Arguments.Item(h)):(WScript.Echo("Wrong arguments - path arguments do not match required format."),WScript.Quit(1))}}dir_assignment.sort(function(a,b){return b[0].length-a[0].length}),i=["added","changed","removed"];for(h=0;h<i.length;h++){e=i[h];for(j=0;j<s[e].length;j++)indexOf(s[e][j],s.files)>-1?s[e+"_files"].push(s[e][j]):s[e+"_dirs"].push(s[e][j])}k=["option batch abort","option confirm off","open "+winscp_open_arguments,"option transfer binary"],l=movedOrRenamed(s);if(l){for(h=0;h<l.length;h++)k.push('mv "'+remotePath(l[h][0])+'" "'+remotePath(l[h][1])+'"');for(h=0;h<s.changed_files.length;h++)k.push('put "'+s.changed_files[h]+'" "'+remotePath(s.changed_files[h])+'"')}else{s.added_dirs.sort(function(a,b){return a.length-b.length});for(h=0;h<s.added_dirs.length;h++)m=remotePath(s.added_dirs[h]),m!=0&&k.push('mkdir "'+m+'"');for(h=0;h<s.added_files.length;h++)n=remotePath(s.added_files[h]),n!=0&&k.push('put "'+s.added_files[h]+'" "'+n+'"');for(h=0;h<s.changed_files.length;h++)o=remotePath(s.changed_files[h]),o!=0&&k.push('put "'+s.changed_files[h]+'" "'+o+'"');if(explicitly_delete_folder_contents){for(h=0;h<s.removed_files.length;h++)o=remotePath(s.removed_files[h]),o!=0&&k.push('rm "'+o+'"');s.removed_dirs.sort(function(a,b){return b.length-a.length});for(h=0;h<s.removed_dirs.length;h++)p=remotePath(s.removed_dirs[h]),p!=0&&k.push('rmdir "'+p+'"')}else{q=requiredDeletions(s);for(h=0;h<q[0].length;h++)r=remotePath(q[0][h]),r!=0&&k.push('rm "'+r+'"');for(h=0;h<q[1].length;h++)r=remotePath(q[1][h]),r!=0&&k.push('rmdir "'+r+'"')}}return k.push("exit"),execWinSCPCommands(k)}function sync(){var a,b;dir_assignment.sort(function(a,b){return a[0].length-b[0].length}),a=["option batch abort","option confirm off","open "+winscp_open_arguments,"option transfer binary"];for(b=0;b<dir_assignment.length;b++)a.push('synchronize remote -delete -criteria=either "'+project_dir+dir_assignment[b][0]+'" "'+dir_assignment[b][1]+'"');return a.push("exit"),execWinSCPCommands(a)}function movedOrRenamed(a){var b,c,d,e,f,g,h,i,j,k,l;if(a.added.length!=0&&a.added.length==a.removed.length){a.added.sort(function(a,b){return a.length-b.length}),a.removed.sort(function(a,b){return a.length-b.length}),b=a.added[0].split("\\"),c=b.length,b=b.slice(0,c-1).join("\\"),d=b.length,e=a.removed[0].split("\\"),f=e.length,e=e.slice(0,f-1).join("\\"),g=e.length;for(h=1;h<a.added.length;h++)if(a.added[h].substring(d)!=a.removed[h].substring(g))return!1;if(remotePath(b)==0||remotePath(e)==0)return!1;i=[];for(h=0;h<a.added.length;h++){j=[a.removed[h].split("\\").slice(0,f).join("\\"),a.added[h].split("\\").slice(0,c).join("\\")],k=!1;for(l=0;l<i.length;l++)if(i[l][0]==j[0]){k=!0;break}k||i.push(j)}return i}return!1}function requiredDeletions(a){var b,c,d,e;a.removed_dirs.sort(function(a,b){return a.length-b.length}),b=[[],[]];for(c=0;c<a.removed_files.length;c++){d=!1;for(e=0;e<a.removed_dirs.length;e++)if(a.removed_files[c].substr(0,a.removed_dirs[e].length+1)==a.removed_dirs[e]+"\\"){d=!0;break}d||b[0].push(a.removed_files[c])}for(c=a.removed_dirs.length-1;c>=0;c--){d=!1;for(e=0;e<c;e++)if(a.removed_dirs[c].substr(0,a.removed_dirs[e].length+1)==a.removed_dirs[e]+"\\"){d=!0;break}d||b[1].push(a.removed_dirs[c])}return b}function indexOf(a,b){var c;for(c=0;c<b.length;c++)if(b[c]===a)return c;return-1}function remotePath(a){var b,c;a=a.substring(project_dir.length),b="";for(c=0;c<dir_assignment.length;c++){b=a.substr(0,dir_assignment[c][0].length);if(b==dir_assignment[c][0])return dir_assignment[c][1]+a.substring(b.length).replace(/\\/g,"/")}return!1}function getWinSCPScriptName(){var a=new Date,b=a.getMonth()+1,c=a.getDate(),d=a.getHours(),e=a.getMinutes(),f=a.getSeconds();return b<10&&(b="0"+b),c<10&&(c="0"+c),d<10&&(d="0"+d),e<10&&(e="0"+e),f<10&&(f="0"+f),"autobuild_"+b+"-"+c+"-"+a.getFullYear()+"_"+d+"-"+e+"-"+f+".txt"}function execWinSCPCommands(a){var b,c,d,e,f,g,h,i,j,k,l,m=!1;if(!winscp_console_mode){b=getWinSCPScriptName(),c=new ActiveXObject("Scripting.FilesystemObject"),d=c.CreateTextFile(b,!0);for(e=0;e<a.length;e++)d.WriteLine(a[e]);d.Close(),f=WScript.CreateObject("WScript.Shell"),WScript.Echo("Starting WinSCP..."),g=f.Exec('"'+winscp_com+'" /script='+b);while(!g.StdOut.AtEndOfStream)h=g.StdOut.ReadLine(),WScript.Echo(h),h.substring(h.length-8)==" failed."&&(m=!0);c.FileExists(b)&&c.DeleteFile(b)}else{f=WScript.CreateObject("WScript.Shell"),WScript.Echo("Starting WinSCP..."),g=f.Exec(winscp_com),i="",j="",k=0,l=!1,e=0;while(!g.StdOut.AtEndOfStream){j=g.StdOut.Read(1),l||WScript.StdOut.Write(j);if(j=="\n"){if(i.substring(i.length-9)==" failed.\r"||i.substring(i.length-8)==" failed.")m=!0;i=""}else i+=j,k==1&&i.substring(i.length-3)=="://"&&j!="\n"?(i="",k=2):k==2&&i.substring(i.length-1)==":"&&j!="\n"?(l=!0,WScript.StdOut.Write("***"),i="",k=3):k==3&&i.substring(i.length-1)=="@"&&j!="\n"?(l=!1,WScript.StdOut.Write("@"),k=0):i=="winscp> "&&(a[e].substr(0,5)=="open "?k=1:k=0,g.StdIn.Write(a[e]+"\r\n"),e++)}}return g.ExitCode==0&&!m?WScript.Echo("WinSCP finished without errors."):g.ExitCode==0&&m?WScript.Echo("WinSCP finished with errors."):WScript.Echo("WinSCP finished with errors - exit code "+g.ExitCode+"."),g.ExitCode==0&&m?1:g.ExitCode}var action,project_dir,autobuild_result;WScript.Arguments.length<2?(WScript.Echo("Wrong arguments - no action/project dir specified."),WScript.Quit(1)):(action=WScript.Arguments.Item(0),project_dir=WScript.Arguments.Item(1)),action=="-autobuild"?(autobuild_result=autobuild(),autobuild_result!=0&&synchronize_on_autobuild_failure?(WScript.Echo("autobuild failed ("+autobuild_result+"), starting full sync..."),WScript.Quit(sync())):WScript.Quit(autobuild_result)):action=="-sync"?WScript.Quit(sync()):(WScript.Echo('Wrong arguments - unknown action "'+action+'".'),WScript.Quit(1));